@@ -31,6 +31,7 @@
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/simple-password-manager.h>
 #include <telepathy-glib/svc-connection.h>
 #include <telepathy-glib/channel-manager.h>
 
@@ -122,6 +123,7 @@ enum {
 	PROP_CHARSET,
 	PROP_QUITMESSAGE,
 	PROP_USE_SSL,
+	PROP_PASSWORD_PROMPT,
 	LAST_PROPERTY_ENUM
 };
 
@@ -145,6 +147,7 @@ struct _IdleConnectionPrivate {
 	char *charset;
 	char *quit_message;
 	gboolean use_ssl;
+	gboolean password_prompt;
 
 	/* the string used by the a server as a prefix to any messages we send that
 	 * it relays to other users.  We need to know this so we can keep our sent
@@ -171,6 +174,9 @@ struct _IdleConnectionPrivate {
 
 	/* if idle_connection_dispose has already run once */
 	gboolean dispose_has_run;
+
+	/* so we can pop up a SASL channel asking for the password */
+	TpSimplePasswordManager *password_manager;
 };
 
 #define IDLE_CONNECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IDLE_TYPE_CONNECTION, IdleConnectionPrivate))
@@ -262,6 +268,10 @@ static void idle_connection_set_property(GObject *obj, guint prop_id, const GVal
 			priv->use_ssl = g_value_get_boolean(value);
 			break;
 
+		case PROP_PASSWORD_PROMPT:
+			priv->password_prompt = g_value_get_boolean(value);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
 			break;
@@ -308,6 +318,10 @@ static void idle_connection_get_property(GObject *obj, guint prop_id, GValue *va
 			g_value_set_boolean(value, priv->use_ssl);
 			break;
 
+		case PROP_PASSWORD_PROMPT:
+			g_value_set_boolean(value, priv->password_prompt);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
 			break;
@@ -364,15 +378,21 @@ static void idle_connection_finalize (GObject *object) {
 	G_OBJECT_CLASS(idle_connection_parent_class)->finalize(object);
 }
 
+static const gchar * interfaces_always_present[] = {
+	TP_IFACE_CONNECTION_INTERFACE_ALIASING,
+	IDLE_IFACE_CONNECTION_INTERFACE_RENAMING,
+	TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
+	NULL};
+
+const gchar * const *idle_connection_get_implemented_interfaces (void) {
+	/* we don't have any conditionally-implemented interfaces yet */
+	return interfaces_always_present;
+}
+
 static void idle_connection_class_init(IdleConnectionClass *klass) {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	TpBaseConnectionClass *parent_class = TP_BASE_CONNECTION_CLASS(klass);
 	GParamSpec *param_spec;
-	static const gchar *interfaces_always_present[] = {
-		TP_IFACE_CONNECTION_INTERFACE_ALIASING,
-		IDLE_IFACE_CONNECTION_INTERFACE_RENAMING,
-		TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
-		NULL};
 
 	g_type_class_add_private(klass, sizeof(IdleConnectionPrivate));
 
@@ -393,35 +413,39 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	parent_class->start_connecting = _iface_start_connecting;
 	parent_class->interfaces_always_present = interfaces_always_present;
 
-	param_spec = g_param_spec_string("nickname", "IRC nickname", "The nickname to be visible to others in IRC.", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+	param_spec = g_param_spec_string("nickname", "IRC nickname", "The nickname to be visible to others in IRC.", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_NICKNAME, param_spec);
 
-	param_spec = g_param_spec_string("server", "Hostname or IP of the IRC server to connect to", "The server used when establishing the connection.", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+	param_spec = g_param_spec_string("server", "Hostname or IP of the IRC server to connect to", "The server used when establishing the connection.", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_SERVER, param_spec);
 
-	param_spec = g_param_spec_uint("port", "IRC server port", "The destination port used when establishing the connection.", 0, G_MAXUINT16, 0, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT);
+	param_spec = g_param_spec_uint("port", "IRC server port", "The destination port used when establishing the connection.", 0, G_MAXUINT16, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_PORT, param_spec);
 
-	param_spec = g_param_spec_string("password", "Server password", "Password to authenticate to the server with", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+	param_spec = g_param_spec_string("password", "Server password", "Password to authenticate to the server with", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_PASSWORD, param_spec);
 
-	param_spec = g_param_spec_string("realname", "Real name", "The real name of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+	param_spec = g_param_spec_string("realname", "Real name", "The real name of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_REALNAME, param_spec);
 
-	param_spec = g_param_spec_string("username", "User name", "The username of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+	param_spec = g_param_spec_string("username", "User name", "The username of the user connecting to IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	g_object_class_install_property(object_class, PROP_USERNAME, param_spec);
 
-	param_spec = g_param_spec_string("charset", "Character set", "The character set to use to communicate with the outside world", "NULL", G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT);
+	param_spec = g_param_spec_string("charset", "Character set", "The character set to use to communicate with the outside world", "NULL", G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_CHARSET, param_spec);
 
-	param_spec = g_param_spec_string("quit-message", "Quit message", "The quit message to send to the server when leaving IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT);
+	param_spec = g_param_spec_string("quit-message", "Quit message", "The quit message to send to the server when leaving IRC", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_QUITMESSAGE, param_spec);
 
-	param_spec = g_param_spec_boolean("use-ssl", "Use SSL", "If the connection should use a SSL tunneled socket connection", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT);
+	param_spec = g_param_spec_boolean("use-ssl", "Use SSL", "If the connection should use a SSL tunneled socket connection", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_USE_SSL, param_spec);
+
+	param_spec = g_param_spec_boolean("password-prompt", "Password prompt", "Whether the connection should pop up a SASL channel if no password is given", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
+	g_object_class_install_property(object_class, PROP_PASSWORD_PROMPT, param_spec);
 }
 
 static GPtrArray *_iface_create_channel_managers(TpBaseConnection *self) {
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(self);
 	GPtrArray *managers = g_ptr_array_sized_new(1);
 	GObject *manager;
 
@@ -431,6 +455,9 @@ static GPtrArray *_iface_create_channel_managers(TpBaseConnection *self) {
 	manager = g_object_new(IDLE_TYPE_MUC_MANAGER, "connection", self, NULL);
 	g_ptr_array_add(managers, manager);
 
+	priv->password_manager = tp_simple_password_manager_new(self);
+	g_ptr_array_add(managers, priv->password_manager);
+
 	return managers;
 }
 
@@ -472,6 +499,12 @@ static void _iface_disconnected(TpBaseConnection *self) {
 	IdleConnection *conn = IDLE_CONNECTION(self);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 
+	/* we never got around to actually creating the connection
+         * iface object, so don't try to send any traffic down it */
+	if (priv->conn == NULL) {
+		return;
+	}
+
 	send_quit_request(conn);
 
 	priv->quitting = TRUE;
@@ -497,6 +530,37 @@ static void _iface_shut_down(TpBaseConnection *self) {
 	}
 }
 
+static void _start_connecting_continue(IdleConnection *conn);
+
+static void _password_prompt_cb(GObject *source, GAsyncResult *result, gpointer user_data) {
+	IdleConnection *conn = user_data;
+	TpBaseConnection *base_conn = TP_BASE_CONNECTION(conn);
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	const GString *password;
+	GError *error = NULL;
+
+	password = tp_simple_password_manager_prompt_finish(TP_SIMPLE_PASSWORD_MANAGER(source), result, &error);
+
+	if (error != NULL) {
+		IDLE_DEBUG("Simple password manager failed: %s", error->message);
+
+		if (base_conn->status != TP_CONNECTION_STATUS_DISCONNECTED) {
+			tp_base_connection_disconnect_with_dbus_error(base_conn,
+								      tp_error_get_dbus_name(error->code),
+								      NULL,
+								      TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED);
+		}
+
+		g_error_free(error);
+		return;
+	}
+
+	g_free(priv->password);
+	priv->password = g_strdup(password->str);
+
+	_start_connecting_continue(conn);
+}
+
 static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error) {
 	IdleConnection *conn = IDLE_CONNECTION(self);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
@@ -505,67 +569,77 @@ static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error)
 	g_assert(priv->server != NULL);
 	g_assert(priv->port > 0 && priv->port <= G_MAXUINT16);
 
-	if (priv->conn == NULL) {
-		GError *conn_error = NULL;
-		IdleServerConnectionIface *sconn;
-		GType connection_type = (priv->use_ssl) ? IDLE_TYPE_SSL_SERVER_CONNECTION : IDLE_TYPE_SERVER_CONNECTION;
-
-		if (!priv->realname || !priv->realname[0]) {
-			const gchar *g_realname = g_get_real_name();
-
-			g_free(priv->realname);
-
-			if (g_realname && g_realname[0] && strcmp(g_realname, "Unknown"))
-				priv->realname = g_strdup(g_realname);
-			else
-				priv->realname = g_strdup(priv->nickname);
-		}
-
-		if (!priv->username || !priv->username[0]) {
-			g_free(priv->username);
-			priv->username = g_strdup(priv->nickname);
-		}
-
-		sconn = IDLE_SERVER_CONNECTION_IFACE(g_object_new(connection_type, "host", priv->server, "port", priv->port, NULL));
-
-		g_signal_connect(sconn, "status-changed", (GCallback)(sconn_status_changed_cb), conn);
-
-		if (!idle_server_connection_iface_connect(sconn, &conn_error)) {
-			IDLE_DEBUG("server connection failed to connect: %s", conn_error->message);
-			g_set_error(error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, "failed to open low-level network connection: %s", conn_error->message);
-
-			g_error_free(conn_error);
-			g_object_unref(sconn);
-
-			return FALSE;
-		}
-
-		priv->conn = sconn;
-
-		g_signal_connect(sconn, "received", (GCallback)(sconn_received_cb), conn);
-
-		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_ERRONEOUSNICKNAME, _erroneous_nickname_handler, conn);
-		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_NICKNAMEINUSE, _nickname_in_use_handler, conn);
-		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WELCOME, _welcome_handler, conn);
-		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WHOISUSER, _whois_user_handler, conn);
-
-		idle_parser_add_handler(conn->parser, IDLE_PARSER_CMD_PING, _ping_handler, conn);
-
-		idle_parser_add_handler_with_priority(conn->parser, IDLE_PARSER_PREFIXCMD_NICK, _nick_handler, conn, IDLE_PARSER_HANDLER_PRIORITY_FIRST);
-		idle_parser_add_handler(conn->parser, IDLE_PARSER_PREFIXCMD_PRIVMSG_USER, _version_privmsg_handler, conn);
-
-		irc_handshakes(conn);
-	} else {
+	if (priv->conn != NULL) {
 		IDLE_DEBUG("conn already open!");
-
 		g_set_error(error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE, "connection already open!");
-
 		return FALSE;
 	}
 
+	if (priv->password_prompt) {
+		tp_simple_password_manager_prompt_async(priv->password_manager, _password_prompt_cb, conn);
+	} else {
+		_start_connecting_continue(conn);
+	}
+
 	return TRUE;
 }
 
+static void _start_connecting_continue(IdleConnection *conn) {
+	TpBaseConnection *base_conn = TP_BASE_CONNECTION(conn);
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	GError *conn_error = NULL;
+	IdleServerConnectionIface *sconn;
+	GType connection_type = (priv->use_ssl) ? IDLE_TYPE_SSL_SERVER_CONNECTION : IDLE_TYPE_SERVER_CONNECTION;
+
+	if (!priv->realname || !priv->realname[0]) {
+		const gchar *g_realname = g_get_real_name();
+
+		g_free(priv->realname);
+
+		if (g_realname && g_realname[0] && strcmp(g_realname, "Unknown"))
+			priv->realname = g_strdup(g_realname);
+		else
+			priv->realname = g_strdup(priv->nickname);
+	}
+
+	if (!priv->username || !priv->username[0]) {
+		g_free(priv->username);
+		priv->username = g_strdup(priv->nickname);
+	}
+
+	sconn = IDLE_SERVER_CONNECTION_IFACE(g_object_new(connection_type, "host", priv->server, "port", priv->port, NULL));
+
+	g_signal_connect(sconn, "status-changed", (GCallback)(sconn_status_changed_cb), conn);
+
+	if (!idle_server_connection_iface_connect(sconn, &conn_error)) {
+		IDLE_DEBUG("server connection failed to connect: %s", conn_error->message);
+
+		tp_base_connection_disconnect_with_dbus_error(base_conn,
+							      TP_ERROR_STR_NETWORK_ERROR,
+							      NULL,
+							      TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
+
+		g_object_unref(sconn);
+		return;
+	}
+
+	priv->conn = sconn;
+
+	g_signal_connect(sconn, "received", (GCallback)(sconn_received_cb), conn);
+
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_ERRONEOUSNICKNAME, _erroneous_nickname_handler, conn);
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_NICKNAMEINUSE, _nickname_in_use_handler, conn);
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WELCOME, _welcome_handler, conn);
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WHOISUSER, _whois_user_handler, conn);
+
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_CMD_PING, _ping_handler, conn);
+
+	idle_parser_add_handler_with_priority(conn->parser, IDLE_PARSER_PREFIXCMD_NICK, _nick_handler, conn, IDLE_PARSER_HANDLER_PRIORITY_FIRST);
+	idle_parser_add_handler(conn->parser, IDLE_PARSER_PREFIXCMD_PRIVMSG_USER, _version_privmsg_handler, conn);
+
+	irc_handshakes(conn);
+}
+
 static gboolean msg_queue_timeout_cb(gpointer user_data);
 
 static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn) {
