@@ -100,7 +100,7 @@ static gboolean _channelname_is_valid(const gchar *channel) {
 		return FALSE;
 
 	if (channel[0] == '!') {
-		for (int i = 0; i < 5; i++) {
+		for (gsize i = 0; i < 5 && i + 1 < len; i++) {
 			if (!g_ascii_isupper(channel[i + 1]) && !isdigit(channel[i + 1]))
 				return FALSE;
 		}
@@ -127,7 +127,7 @@ static gboolean _channelname_is_valid(const gchar *channel) {
 	return TRUE;
 }
 
-static gchar *_nick_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpointer ctx, GError **error) {
+gchar *idle_normalize_nickname (const gchar *id, GError **error) {
 	if (!idle_nickname_is_valid(id, FALSE)) {
 		g_set_error(error, TP_ERRORS, TP_ERROR_INVALID_HANDLE, "invalid nickname");
 		return NULL;
@@ -138,6 +138,10 @@ static gchar *_nick_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpo
 	return normalized;
 }
 
+static gchar *_nick_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpointer ctx, GError **error) {
+	return idle_normalize_nickname (id, error);
+}
+
 static gchar *_channel_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpointer ctx, GError **error) {
 	if (!_channelname_is_valid(id)) {
 		g_set_error(error, TP_ERRORS, TP_ERROR_INVALID_HANDLE, "invalid channel ID");
