@@ -45,16 +45,20 @@
 static void _channel_iface_init(gpointer, gpointer);
 static void _password_iface_init(gpointer, gpointer);
 static void _properties_iface_init(gpointer, gpointer);
-static void _text_iface_init(gpointer, gpointer);
+
+static void idle_muc_channel_send (GObject *obj, TpMessage *message, TpMessageSendingFlags flags);
 
 G_DEFINE_TYPE_WITH_CODE(IdleMUCChannel, idle_muc_channel, G_TYPE_OBJECT,
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL, _channel_iface_init);
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP, tp_group_mixin_iface_init);
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_INTERFACE_PASSWORD, _password_iface_init);
-		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_TYPE_TEXT, _text_iface_init);
+		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_TYPE_TEXT, tp_message_mixin_text_iface_init);
+		G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_MESSAGES, tp_message_mixin_messages_iface_init);
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_PROPERTIES_INTERFACE, _properties_iface_init);
 		G_IMPLEMENT_INTERFACE(TP_TYPE_EXPORTABLE_CHANNEL, NULL);
-		G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_IFACE, NULL);)
+		G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_IFACE, NULL);
+		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_DBUS_PROPERTIES, tp_dbus_properties_mixin_iface_init);)
+
 
 /* signal enum */
 enum {
@@ -112,6 +116,8 @@ typedef enum {
 	LAST_MODE_FLAG_ENUM
 } IRCChannelModeFlags;
 
+#define MODE_FLAGS_OP \
+   (MODE_FLAG_OPERATOR_PRIVILEGE | MODE_FLAG_HALFOP_PRIVILEGE)
 
 typedef struct {
 	IRCChannelModeFlags flags;
@@ -173,6 +179,7 @@ static const gchar *muc_channel_interfaces[] = {
 	TP_IFACE_CHANNEL_INTERFACE_PASSWORD,
 	TP_IFACE_CHANNEL_INTERFACE_GROUP,
 	TP_IFACE_PROPERTIES_INTERFACE,
+	TP_IFACE_CHANNEL_INTERFACE_MESSAGES,
 	NULL
 };
 
@@ -282,9 +289,19 @@ static void idle_muc_channel_finalize (GObject *object);
 static GObject *idle_muc_channel_constructor(GType type, guint n_props, GObjectConstructParam *props) {
 	GObject *obj = G_OBJECT_CLASS(idle_muc_channel_parent_class)->constructor(type, n_props, props);
 	IdleMUCChannelPrivate *priv = IDLE_MUC_CHANNEL_GET_PRIVATE(obj);
-	DBusGConnection *bus;
-	TpHandleRepoIface *room_handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->connection), TP_HANDLE_TYPE_ROOM);
-	TpHandleRepoIface *contact_handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->connection), TP_HANDLE_TYPE_CONTACT);
+	TpDBusDaemon *bus;
+	TpBaseConnection *conn = TP_BASE_CONNECTION(priv->connection);
+	TpHandleRepoIface *room_handles = tp_base_connection_get_handles(conn, TP_HANDLE_TYPE_ROOM);
+	TpHandleRepoIface *contact_handles = tp_base_connection_get_handles(conn, TP_HANDLE_TYPE_CONTACT);
+	TpChannelTextMessageType types[] = {
+			TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
+			TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
+			TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
+	};
+	const gchar * supported_content_types[] = {
+			"text/plain",
+			NULL
+	};
 
 	tp_handle_ref(room_handles, priv->handle);
 	g_assert(tp_handle_is_valid(room_handles, priv->handle, NULL));
@@ -293,18 +310,19 @@ static GObject *idle_muc_channel_constructor(GType type, guint n_props, GObjectC
 	if (priv->initiator)
 		tp_handle_ref(room_handles, priv->initiator);
 
-	bus = tp_get_bus();
-	dbus_g_connection_register_g_object(bus, priv->object_path, obj);
+	bus = tp_base_connection_get_dbus_daemon (conn);
+	tp_dbus_daemon_register_object (bus, priv->object_path, obj);
 
 	tp_group_mixin_init(obj, G_STRUCT_OFFSET(IdleMUCChannel, group), contact_handles, priv->connection->parent.self_handle);
 	tp_group_mixin_change_flags(obj, TP_CHANNEL_GROUP_FLAG_PROPERTIES, 0);
 
-	tp_text_mixin_init(obj, G_STRUCT_OFFSET(IdleMUCChannel, text), contact_handles);
-	tp_text_mixin_set_message_types(obj,
-			TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
-			TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
-			TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
-			G_MAXUINT);
+	/* initialize message mixin */
+	tp_message_mixin_init (obj, G_STRUCT_OFFSET (IdleMUCChannel, message_mixin),
+			conn);
+	tp_message_mixin_implement_sending (obj, idle_muc_channel_send,
+			G_N_ELEMENTS (types), types, 0,
+			TP_DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES,
+			supported_content_types);
 
 	if (priv->requested) {
 		/* Add ourself to 'remote-pending' while we are joining the channel */
@@ -512,7 +530,7 @@ static void idle_muc_channel_class_init (IdleMUCChannelClass *idle_muc_channel_c
 	signals[JOIN_READY] = g_signal_new("join-ready", G_OBJECT_CLASS_TYPE(idle_muc_channel_class), G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, 0, NULL, NULL, g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1, G_TYPE_UINT);
 
 	tp_group_mixin_class_init(object_class, G_STRUCT_OFFSET(IdleMUCChannelClass, group_class), add_member, remove_member);
-	tp_text_mixin_class_init(object_class, G_STRUCT_OFFSET(IdleMUCChannelClass, text_class));
+	tp_message_mixin_init_dbus_properties (object_class);
 
 	static TpDBusPropertiesMixinPropImpl channel_props[] = {
 		{ "Interfaces", "interfaces", NULL },
@@ -580,8 +598,8 @@ void idle_muc_channel_finalize (GObject *object) {
 	if (priv->namereply_set)
 		tp_handle_set_destroy(priv->namereply_set);
 
-	tp_text_mixin_finalize(object);
 	tp_group_mixin_finalize(object);
+	tp_message_mixin_finalize (object);
 
 	G_OBJECT_CLASS (idle_muc_channel_parent_class)->finalize (object);
 }
@@ -613,7 +631,10 @@ static void muc_channel_tp_properties_init(IdleMUCChannel *chan) {
 
 		g_value_init(value, property_signatures[i].type);
 
-		props[i].flags = 0;
+		if (i == TP_PROPERTY_SUBJECT)
+			props[i].flags = TP_PROPERTY_FLAG_WRITE;
+		else
+			props[i].flags = 0;
 	}
 }
 
@@ -950,7 +971,10 @@ static void change_mode_state(IdleMUCChannel *obj, guint add, guint remove) {
 
 		for (int i = 0; flags_helper[i] != LAST_TP_PROPERTY_ENUM; i++) {
 			guint prop_id = flags_helper[i];
-			g_array_append_val(flags_to_change, prop_id);
+			/* Only handle subject flags here if it's op setable only */
+			if (prop_id != TP_PROPERTY_SUBJECT ||
+					(flags & MODE_FLAG_TOPIC_ONLY_SETTABLE_BY_OPS))
+				g_array_append_val(flags_to_change, prop_id);
 		}
 
 		prop_flags = TP_PROPERTY_FLAG_WRITE;
@@ -967,6 +991,20 @@ static void change_mode_state(IdleMUCChannel *obj, guint add, guint remove) {
 
 			set_tp_property_flags(obj, flags_to_change, 0, prop_flags);
 		}
+	} else if ((combined & MODE_FLAG_TOPIC_ONLY_SETTABLE_BY_OPS)
+			&& !(priv->mode_state.flags & MODE_FLAGS_OP)) {
+		/* We're not ops and the MODE_FLAG_TOPIC_ONLY_SETTABLE_BY_OPS flag was
+		 * changed */
+		GArray *flags_to_change;
+		guint prop_id = TP_PROPERTY_SUBJECT;
+
+		flags_to_change = g_array_new(FALSE, FALSE, sizeof(guint));
+		g_array_append_val(flags_to_change, prop_id);
+
+		if (add & MODE_FLAG_TOPIC_ONLY_SETTABLE_BY_OPS)
+			set_tp_property_flags(obj, flags_to_change, 0, TP_PROPERTY_FLAG_WRITE);
+		else
+			set_tp_property_flags(obj, flags_to_change, TP_PROPERTY_FLAG_WRITE, 0);
 	}
 
 	for (int i = 1; i < LAST_MODE_FLAG_ENUM; i <<= 1) {
@@ -1066,12 +1104,10 @@ static void change_password_flags(IdleMUCChannel *obj, guint flag, gboolean stat
 }
 
 gboolean idle_muc_channel_receive(IdleMUCChannel *chan, TpChannelTextMessageType type, TpHandle sender, const gchar *text) {
-	time_t stamp = time(NULL);
+	IdleMUCChannelPrivate *priv = IDLE_MUC_CHANNEL_GET_PRIVATE (chan);
+	TpBaseConnection *base_conn = TP_BASE_CONNECTION (priv->connection);
 
-	g_assert(chan != NULL);
-	g_assert(IDLE_IS_MUC_CHANNEL(chan));
-
-	return tp_text_mixin_receive(G_OBJECT(chan), type, sender, stamp, text);
+	return idle_text_received (G_OBJECT (chan), base_conn, type, text, sender);
 }
 
 static void send_mode_query_request(IdleMUCChannel *chan) {
@@ -2039,27 +2075,25 @@ static void idle_muc_channel_provide_password (TpSvcChannelInterfacePassword *if
 /**
  * idle_muc_channel_send
  *
- * Implements DBus method Send
- * on interface org.freedesktop.Telepathy.Channel.Type.Text
- *
- * @error: Used to return a pointer to a GError detailing any error
- *         that occured, DBus will throw the error only if this
- *         function returns false.
- *
- * Returns: TRUE if successful, FALSE if an error was thrown.
+ * Indirectly implements (via TpMessageMixin) D-Bus method Send on interface
+ * org.freedesktop.Telepathy.Channel.Type.Text and D-Bus method SendMessage on
+ * Channel.Interface.Messages
  */
-static void idle_muc_channel_send (TpSvcChannelTypeText *iface, guint type, const gchar * text, DBusGMethodInvocation *context) {
-	IdleMUCChannel *obj = (IdleMUCChannel *)(iface);
-	IdleMUCChannelPrivate *priv = IDLE_MUC_CHANNEL_GET_PRIVATE(obj);
-	time_t timestamp = time(NULL);
+static void
+idle_muc_channel_send (GObject *obj, TpMessage *message, TpMessageSendingFlags flags)
+{
+	IdleMUCChannel *self = (IdleMUCChannel *) obj;
+	IdleMUCChannelPrivate *priv = IDLE_MUC_CHANNEL_GET_PRIVATE(self);
 
 	if ((priv->mode_state.flags & MODE_FLAG_MODERATED) && !(priv->mode_state.flags & (MODE_FLAG_OPERATOR_PRIVILEGE | MODE_FLAG_HALFOP_PRIVILEGE | MODE_FLAG_VOICE_PRIVILEGE))) {
-		IDLE_DEBUG("emitting SEND_ERROR with (%u, %" G_GUINT64_FORMAT ", %u, %s)", TP_CHANNEL_TEXT_SEND_ERROR_PERMISSION_DENIED, (guint64)(timestamp), type, text);
-		tp_svc_channel_type_text_emit_send_error(iface, TP_CHANNEL_TEXT_SEND_ERROR_PERMISSION_DENIED, timestamp, type, text);
+		GError error = { TP_ERRORS, TP_ERROR_PERMISSION_DENIED, "Channel is moderated" };
+
+		IDLE_DEBUG("Channel is moderated");
+		tp_message_mixin_sent (obj, message, 0, NULL, &error);
 		return;
 	}
 
-	idle_text_send((GObject *)(obj), type, priv->channel_name, text, priv->connection, context);
+	idle_text_send(obj, message, flags, priv->channel_name, priv->connection);
 }
 
 static char to_irc_mode(IdleMUCChannelTPProperty prop_id) {
@@ -2370,14 +2404,3 @@ static void _properties_iface_init(gpointer g_iface, gpointer iface_data) {
 	IMPLEMENT(set_properties);
 #undef IMPLEMENT
 }
-
-static void _text_iface_init(gpointer g_iface, gpointer iface_data) {
-	TpSvcChannelTypeTextClass *klass = (TpSvcChannelTypeTextClass *)(g_iface);
-
-	tp_text_mixin_iface_init(g_iface, iface_data);
-#define IMPLEMENT(x) tp_svc_channel_type_text_implement_##x (\
-		klass, idle_muc_channel_##x)
-	IMPLEMENT(send);
-#undef IMPLEMENT
-}
-
