@@ -82,7 +82,7 @@ static void _muc_manager_close_all(IdleMUCManager *manager);
 static void _muc_manager_add_handlers(IdleMUCManager *manager);
 
 static void _muc_manager_foreach_channel(TpChannelManager *manager, TpExportableChannelFunc func, gpointer user_data);
-static void _muc_manager_foreach_channel_class (TpChannelManager *manager, TpChannelManagerChannelClassFunc func, gpointer user_data);
+static void _muc_manager_type_foreach_channel_class (GType type, TpChannelManagerTypeChannelClassFunc func, gpointer user_data);
 
 static gboolean _muc_manager_create_channel (TpChannelManager *manager, gpointer request_token, GHashTable *request_properties);
 static gboolean _muc_manager_request_channel (TpChannelManager *manager, gpointer request_token, GHashTable *request_properties);
@@ -602,8 +602,8 @@ static void _muc_manager_foreach_channel(TpChannelManager *iface, TpExportableCh
 }
 
 static void
-_muc_manager_foreach_channel_class (TpChannelManager *manager,
-									TpChannelManagerChannelClassFunc func,
+_muc_manager_type_foreach_channel_class (GType type,
+									TpChannelManagerTypeChannelClassFunc func,
 									gpointer user_data)
 {
 	GHashTable *table = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -620,7 +620,7 @@ _muc_manager_foreach_channel_class (TpChannelManager *manager,
 	g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType",
 						 handle_type_value);
 
-	func (manager, table, muc_channel_allowed_properties,
+	func (type, table, muc_channel_allowed_properties,
 		  user_data);
 
 	g_hash_table_destroy (table);
@@ -853,7 +853,7 @@ static void _muc_manager_iface_init(gpointer g_iface, gpointer iface_data) {
 	TpChannelManagerIface *iface = g_iface;
 
 	iface->foreach_channel = _muc_manager_foreach_channel;
-	iface->foreach_channel_class = _muc_manager_foreach_channel_class;
+	iface->type_foreach_channel_class = _muc_manager_type_foreach_channel_class;
 	iface->request_channel = _muc_manager_request_channel;
 	iface->create_channel = _muc_manager_create_channel;
 	iface->ensure_channel = _muc_manager_ensure_channel;
