@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -28,13 +28,11 @@ G_BEGIN_DECLS
 typedef struct _IdleServerConnection IdleServerConnection;
 typedef struct _IdleServerConnectionClass IdleServerConnectionClass;
 
-struct _IdleServerConnection
-{
+struct _IdleServerConnection {
 	GObject parent;
 };
 
-struct _IdleServerConnectionClass
-{
+struct _IdleServerConnectionClass {
 	GObjectClass parent;
 };
 
